/*********************************************************************
*                                                                    *          
*                              FLY                                   *
*                           --E#Y#E--							     *
*                         ===MUSCLE===                               *
*                                                                    *
*********************************************************************/
/* Header Files */
#include "opengl.h"
/* Old Style */ 
#define UNI

/* Load File */
FILE *gridfi;
FILE *forest;
FILE *desert;
FILE *seaoce;
/* Texture */
GLubyte *bla;
GLubyte *des;
GLubyte *sky;
GLubyte *sea;
/* GLfloat */
GLfloat roll;
GLfloat yaw;
GLfloat pitch;
GLfloat planeX;
GLfloat planeY;
GLfloat planeZ;
GLfloat rootY;
GLfloat viewZ;
/* RGB Colour */
char code;
float VA0;
float VA1;    
float VA5;
float VA10;
/* Render Texture */
float projection[ID];
float modelview[ID];

/*---------------------------------------------------------------------------*/    

/* Load Textures */
void loadPNG()
{
	/* Filename */	
	char *filename1;
	char *filename2;
	char *filename3;
	char *filename4;

	/* Load Textures */	
	filename1 = "C/PIXEL/forest.png";
	read_png(filename1, &bla);		
	filename2 = "C/PIXEL/desert.png";
	read_png(filename2, &des);
	filename3 = "C/PIXEL/cloud.png";
	read_png(filename3, &sky);
	filename4 = "C/PIXEL/sea.png";
	read_png(filename4, &sea);
}

/*---------------------------------------------------------------------------*/

/* Init Triangles */
void initGrid()
{	
	/* Background Color */
	glEnable(GL_BLEND);	
	glEnable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	/* Grid */
	glClearColor( 0.1f , 0.1f , 0.1f , 0.5f );
}

/*---------------------------------------------------------------------------*/  

/* Init Triangles */
void initLand(char night)
{	
	/* Background Color */
	glEnable(GL_BLEND);
	glEnable(GL_AMBIENT);
	glEnable(GL_LIGHT7);
	glEnable(GL_LIGHTING);	
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	/* Night */
	if(night)
	{	
		glClearColor( 0.1f , 0.2f , 0.3f , 0.5f );
		glDisable(GL_LIGHT7);	
	}
	/* Day */
	else
	{	
		glClearColor( 0.3f , 0.5f , 0.7f , 0.5f );
		glDisable(GL_LIGHTING);
	}
}

/*---------------------------------------------------------------------------*/

/* Shader Language */
void textureShader()
{
	#ifdef UNI
	/* Texture Shader */
	CreateShader();
	CreateProgram();
	Location(0, "a_Vertex");
	Location(1, "a_TexCoord");

	glEnableVertexAttribArray(0); 
    	glEnableVertexAttribArray(1); 

	/* Passing Data */
	glGetFloatv(GL_PROJECTION_MATRIX, projection);
	glGetFloatv(GL_MODELVIEW_MATRIX, modelview);

	/* Uniform and Matrix */
	Uniform4x4("modelview_matrix", modelview, 0);
	Uniform4x4("projection_matrix", projection, 0);
	Uniform("texture0", 0);

	/* Delete Shader */
	DeleteShader();
	#endif
}

/*---------------------------------------------------------------------------*/

/* Render Grid */
void GridView()
{
	/* Render Triangles */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, 0.0);

	/* Camera Look */
	gluLookAt(0.0f, 2.0f, 20.0f, 
		  0.0f, 2.0f,-20.0f, 
		  0.0f, 5.0f, 0.0f);

	/* Rotation */
	glRotatef(-roll, 0.0f, 0.0f, 1.0f);
	glRotatef(-yaw, 0.0f, 1.0f, 0.0f);
	glRotatef(-pitch, 1.0f, 0.0f, 0.0f);
	glTranslatef(-planeX, -planeY, -planeZ);
	glLineWidth(1);

	/* Landscape File */
	gridfi=fopen("C/DATA/grid.fs","r");	
	
	/* Begin Triangles */
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	glBegin(GL_TRIANGLES);
	/* While Loop */
	glColor4f(0.7f, 0.7f, 0.7f, 1.0f);	
	while(fscanf(gridfi,"%f %f %f",&gl.tri.x,&gl.tri.y,&gl.tri.z)!= EOF)
	{ 
		glVertex3f(gl.tri.x, gl.tri.y, gl.tri.z);
	}
	/* End Triangles */
	glEnd();

	/* File Close */	
	fclose(gridfi);	
}

/*---------------------------------------------------------------------------*/

/* Render Landscape */
void Landscape(char cloud)
{
	/* Render Triangles */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, 0.0);

	/* Camera Look */
	gluLookAt(0.0f, 2.0f, 20.0f, 
		  0.0f, 2.0f,-20.0f, 
		  0.0f, 5.0f, 0.0f);

	/* Rotation */
	glRotatef(-roll, 0.0f, 0.0f, 1.0f);
	glRotatef(-yaw, 0.0f, 1.0f, 0.0f);
	glRotatef(-pitch, 1.0f, 0.0f, 0.0f);
	glTranslatef(-planeX, -planeY, -planeZ);
	glLineWidth(1);

	/* Filtering Mode */
	/*glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);*/
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
	/*glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);*/
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	/* Landscape File */
	forest=fopen("C/DATA/forest.fs","r");
	desert=fopen("C/DATA/desert.fs","r");
	seaoce=fopen("C/DATA/seaoce.fs","r");

	/* Enable Texture */
	glEnable(GL_TEXTURE_2D);
	/* Image Data */
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,128,128,0,GL_BGR,GL_UNSIGNED_BYTE,bla);
	/* Sub Image Data */
	glTexSubImage2D(GL_TEXTURE_2D,0,0,0,128,128,GL_BGR,GL_UNSIGNED_BYTE,bla);	
	glTexSubImage2D(GL_TEXTURE_2D,0,2,0,128,128,GL_BGR,GL_UNSIGNED_BYTE,bla);
	glTexSubImage2D(GL_TEXTURE_2D,0,0,2,128,128,GL_BGR,GL_UNSIGNED_BYTE,bla);
	glTexSubImage2D(GL_TEXTURE_2D,0,2,2,128,128,GL_BGR,GL_UNSIGNED_BYTE,bla);

	/* Begin Triangles */
	glBegin(GL_TRIANGLES);
	/* While Loop */
	while(fscanf(forest,"%s %f %f %f %f %f", &code, &gl.tex.s, &gl.tex.t, 
		&gl.tri.x, &gl.tri.y, &gl.tri.z) != EOF)
	{ 	
		glTexCoord2f(gl.tex.s, gl.tex.t);
		glVertex3f(gl.tri.x, gl.tri.y, gl.tri.z);
	}
	/* End Triangles */
	glEnd();

	/* Image Data */
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,128,128,0,GL_BGRA,GL_UNSIGNED_BYTE,des);
	/* Sub Image Data */
	glTexSubImage2D(GL_TEXTURE_2D,0,0,0,128,128,GL_BGRA,GL_UNSIGNED_BYTE,des);	
	glTexSubImage2D(GL_TEXTURE_2D,0,2,0,128,128,GL_BGRA,GL_UNSIGNED_BYTE,des);
	glTexSubImage2D(GL_TEXTURE_2D,0,0,2,128,128,GL_BGRA,GL_UNSIGNED_BYTE,des);
	glTexSubImage2D(GL_TEXTURE_2D,0,2,2,128,128,GL_BGRA,GL_UNSIGNED_BYTE,des);

	/* Begin Triangles */
	glBegin(GL_TRIANGLES);
	/* While Loop */
	while(fscanf(desert,"%s %f %f %f %f %f", &code, &gl.tex.s, &gl.tex.t, 
		&gl.tri.x, &gl.tri.y, &gl.tri.z) != EOF)
	{ 	
		glTexCoord2f(gl.tex.s, gl.tex.t);
		glVertex3f(gl.tri.x, gl.tri.y, gl.tri.z);
	}	
	/* End Triangles */
	glEnd();

	/* Image Data */
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,128,128,0,GL_BGRA,GL_UNSIGNED_BYTE,sea);
	/* Sub Image Data */
	glTexSubImage2D(GL_TEXTURE_2D,0,0,0,128,128,GL_BGRA,GL_UNSIGNED_BYTE,sea);	
	glTexSubImage2D(GL_TEXTURE_2D,0,2,0,128,128,GL_BGRA,GL_UNSIGNED_BYTE,sea);
	glTexSubImage2D(GL_TEXTURE_2D,0,0,2,128,128,GL_BGRA,GL_UNSIGNED_BYTE,sea);
	glTexSubImage2D(GL_TEXTURE_2D,0,2,2,128,128,GL_BGRA,GL_UNSIGNED_BYTE,sea);

	/* Begin Triangles */
	glBegin(GL_TRIANGLES);
	/* While Loop */
	while(fscanf(seaoce,"%s %f %f %f %f %f", &code, &gl.tex.s, &gl.tex.t, 
		&gl.tri.x, &gl.tri.y, &gl.tri.z) != EOF)
	{ 	
		glTexCoord2f(gl.tex.s, gl.tex.t);
		glVertex3f(gl.tri.x, gl.tri.y, gl.tri.z);
	}	
	/* End Triangles */
	glEnd();
	
	/* Texture Cloud */
	if(cloud)
	{
		/* Image Data */
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,128,128,0,
				GL_BGRA,GL_UNSIGNED_BYTE,sky);
		/* Clouds */
		glBegin(GL_QUADS);
		glTexCoord2f(0.0,0.0);glVertex3f( 20.0f,4.0f, 20.0f);
		glTexCoord2f(2.0,0.0);glVertex3f( 20.0f,4.0f,-20.0f);
		glTexCoord2f(2.0,2.0);glVertex3f(-20.0f,4.0f,-20.0f);
		glTexCoord2f(0.0,2.0);glVertex3f(-20.0f,4.0f, 20.0f);
		glEnd();
	}
	
	/* File Close */	
	fclose(forest);
	fclose(desert);
	fclose(seaoce);
	/* Disable Texture */
	glDisable(GL_COLOR_MATERIAL);	
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glDisable(GL_AMBIENT);
}

/*-------------------------------------------------------------------------*/

/* Render Cockpit */
void Cockpit()
{	
	/* Rotation */
	glRotatef(roll, 0.0f, 0.0f ,1.0f);
	glRotatef(yaw, 0.0f, 1.0f ,0.0f);
	glRotatef(pitch, 1.0f, 0.0f ,0.0f);
	glTranslatef(planeX, planeY + 2.0f, planeZ + viewZ + 18.499f);
	glLineWidth(2);	

	/* Render Cockpit */
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	/* Begin Quads */
	glBegin(GL_QUADS);
		/* Cockpit */
		glColor4f( 0.9f, 0.9f, 0.9f, 1.0f);
		glVertex3f(-0.3f, -0.2f, 0.3f );
		glVertex3f( 0.3f, -0.2f, 0.3f );
		glVertex3f( 0.3f,  0.3f, 0.3f );
		glVertex3f(-0.3f,  0.3f, 0.3f );
		
		glColor4f( 0.0 +VA10, 1.0 -VA5, 0.0 +VA0, 1.0 +VA0);		
		glVertex3f(-0.5f, -0.5f, 0.0f );
		glVertex3f( 0.5f, -0.5f, 0.0f );
		glVertex3f( 0.5f,  0.5f, 0.0f );
		glVertex3f(-0.5f,  0.5f, 0.0f );
		
		glVertex3f(-0.7f, -0.5f, 1.5f );
		glVertex3f( 0.7f, -0.5f, 1.5f );
		glVertex3f( 0.5f, -0.5f, 0.0f );
		glVertex3f(-0.5f, -0.5f, 0.0f );
		
		glVertex3f(-0.7f,  0.5f, 1.5f );
		glVertex3f( 0.7f,  0.5f, 1.5f );
		glVertex3f( 0.5f,  0.5f, 0.0f );
		glVertex3f(-0.5f,  0.5f, 0.0f );
		
		/* Terminal */
		glColor4f( 0.9f, 0.9f, 0.9f, 1.0f);	
		glVertex3f(-2.0f, 0.64f + rootY, 0.5f -viewZ );
		glVertex3f( 2.0f, 0.64f + rootY, 0.5f -viewZ );
		glVertex3f( 2.0f, 0.60f + rootY, 0.5f -viewZ );
		glVertex3f(-2.0f, 0.60f + rootY, 0.5f -viewZ );
	/* End Quads */
	glEnd();
	
	/* Render Quads */
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	/* Begin Quads */
	glBegin(GL_QUADS);
		/* Instruments */
		glColor4f(1.0f +VA0 ,1.0f -VA5 ,1.0f -VA10,0.4f -VA1);
		glVertex3f(-0.3f, -0.2f, 0.3f );
		glVertex3f( 0.3f, -0.2f, 0.3f );
		glVertex3f( 0.3f,  0.3f, 0.3f );
		glVertex3f(-0.3f,  0.3f, 0.3f );
		
		glColor4f( 0.2f, 0.2f, 0.2f, 1.0f);
		glVertex3f(-0.7f, -0.5f, 1.5f );
		glVertex3f( 0.7f, -0.5f, 1.5f );
		glVertex3f( 0.5f, -0.5f, 0.0f );
		glVertex3f(-0.5f, -0.5f, 0.0f );
		
		glVertex3f( 0.7f, 0.5f, 1.5f );
		glVertex3f(-0.7f, 0.5f, 1.5f );
		glVertex3f(-0.5f, 0.5f, 0.0f );
		glVertex3f( 0.5f, 0.5f, 0.0f );
		
		/* Terminal */
		glColor4f( 0.2f, 0.3f, 0.4f, 1.0f);
		glVertex3f(-2.0f, 0.60f + rootY, 0.5f -viewZ );
		glVertex3f( 2.0f, 0.60f + rootY, 0.5f -viewZ );
		glVertex3f( 2.0f, 0.64f + rootY, 0.5f -viewZ );
		glVertex3f(-2.0f, 0.64f + rootY, 0.5f -viewZ );

		glColor4f( 0.0f, 0.0f, 0.0f, 1.0f);
		glVertex3d(-2.0f, 0.6f + rootY, 0.5f -viewZ );
		glVertex3d( 2.0f, 0.6f + rootY, 0.5f -viewZ );
		glVertex3d( 2.0f, 1.1f + rootY, 0.5f -viewZ );
		glVertex3d(-2.0f, 1.1f + rootY, 0.5f -viewZ );
		
		/* Night Look */
		glColor4f( 0.0f, 1.0f, 0.0f, 0.2f);
		glVertex3d(-2.0f,-1.0f, 1.0f );
		glVertex3d( 2.0f,-1.0f, 1.0f );
		glVertex3d( 2.0f, 1.0f, 1.0f );
		glVertex3d(-2.0f, 1.0f, 1.0f );
		glColor4f( 1.0f, 1.0f, 1.0f, 1.0f);

	/* End Quads */
	glEnd();
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_BLEND);
}

/*---------------------------------------------------------------------------*/

/* Process Event */
void process_event(XEvent report)
{   
	KeySym key;
	key = XLookupKeysym (&report.xkey, 0);
	
	/* Keyboard Control Keys */	
	switch(key)
	{		
		case XK_Up: 
		pitch -= 0.1f; 
		break;

		case XK_Down: 
		pitch += 0.1f; 
		break;

		case XK_Left: 
		roll += 0.2f; 
		break;

		case XK_Right: 
		roll -= 0.2f; 
		break;

		case XK_e: 
		yaw += 0.1f; 
		break;

		case XK_q: 
		yaw -= 0.1f; 
		break;

		case XK_a: 
		planeX -= 0.02f; 
		break;
		
		case XK_d: 
		planeX += 0.02f; 
		break;

		case XK_w: 
		planeY += 0.01f; 
		break;

		case XK_s: 
		planeY -= 0.01f; 
		break;

		case XK_b: 
		planeZ += 0.02f; 
		break;

		case XK_space: 
		planeZ -= 0.02f; 
		break;

		case XK_m: 
		viewZ = 0.0f; 
		break;

		case XK_n: 
		viewZ = -0.5f; 
		break;

		case XK_F1: 
		rootY = -0.6f; 
		break;

		case XK_F2: 
		rootY = +0.6f; 
		break;

		case XK_F3: 
		VA0 = +0.0f; 
		VA1 = -0.1f;
		VA5 = +0.5f; 
		VA10 = +1.0f; 
		break;

		case XK_F4: 
		VA0 = -0.0f; 
		VA1 = +0.1f;    
		VA5 = -0.5f; 
		VA10 = -1.0f; 
		break;

		case XK_Escape: 
		closeWin(); 
		break;
	}
}

/*---------------------------------------------------------------------------*/

